﻿Encodage des mots de passe pour Mount-Manager
=============================================

Prérequis
---------

Votre système doit supporter `gpg`

```bash
sudo apt update &&
sudo apt install -y gpgv2
```

et vous devez avoir une `clé gpg` ayant comme identificateur `mount-manager`.


Configuration de la clé `gpg`
-----------------------------

### Création automatisée de la clé

Pour plus de détails : [Unattended GPG key generation](https://www.gnupg.org/documentation/manuals/gnupg/Unattended-GPG-key-generation.html) (en anglais)

```bash
cat >foo <<EOF
%echo Generating a basic OpenPGP key

Key-Type: RSA

Key-Length: 4096
Name-Real: mount-manager

# Do a commit here, so that we can later print "done" :-)
%commit
%echo done
EOF
gpg --batch --generate-key foo
```


### Création de la clé en mode interactif

Si l'option `--full-generate-key` n'est pas disponible sur votre système essayez avec `--full-gen-key` voir même si vous avez une très ancienne version `--gen-key`)

```bash
gpg --full-generate-key
```

Pour `mount-manager` la clé est à usage privé et uniquement sur une machine.

> Please select what kind of key you want:

Prendre la réponse `1`: "(1) RSA and RSA (default)"

> What key size do you want? (2048)

Repose `4096`

> Please specify how long the key should be valid.

Définir une durée de clé qui vous semble adaptée dans votre cas. L'option `key does not expire` n'est pas recommandé, mais cela fonctionne.

> Real name:

Pour le nom utilisez obligatoirement `mount-manager`

Puis laissez les autres champs vides.

C’est probablement une *mauvaise idée* de ne pas définir de phrase secrète, mais si vous choisissez cette option pour tester vous pourrez modifier cela par la suite. Vous pouvez d’ailleurs modifier à tout moment votre clé à l’aide de l'option: `--edit-key`.


### Comment modifier la phrase secrète de votre clé

Pour ajouter, changer or retirer la phrase secrète, il suffit d’utiliser la commande :

```bash
gpg --edit-key mount-manager passwd
```


Comment encoder vos mots de passes
----------------------------------

Pour encoder un mot de passe en utilisant votre clé `gpg` utilisez la commande suivante :

```bash
mount-manager-encrypt
```

Vous obtiendrez quelque chose comme :

```json
      "gpg":"-----BEGIN PGP MESSAGE-----^Version: GnuPG v1^^hQIMAznmT ... =uQ1K^-----END PGP MESSAGE-----^",

```

Il vous suffit de copier cette ligne dans la bonne section de votre configuration en utilisant :

```bash
mount-manager-config
```


D'autre commandes `gpg` utiles
------------------------------

Pour obtenir la liste des clés dans votre trousseau public :

```bash
gpg --list-keys
```

Pour obtenir la liste des clés dans votre trousseau secret :

```bash
gpg --list-secret-keys
```

Pour supprimer une clé publique de votre trousseau public :

```bash
gpg --delete-key "User Name or UID"
```

Pour supprimer une clé privée de votre trousseau secret :

```bash
gpg --delete-secret-key "User Name or UID"
```


Références
----------

* <https://help.github.com/articles/generating-a-new-gpg-key/>
* <http://irtfweb.ifa.hawaii.edu/~lockhart/gpg/>

