Mount-Manager password encryption
=================================

Prerequisite
------------

Your system need to support `gpg`

```bash
sudo apt update &&
sudo apt install -y gpgv2
```

And you need a `gpg key` using id `mount-manager`.


`gpg` key configuration
------------------------

### Unattended key generation

More details: [Unattended GPG key generation](https://www.gnupg.org/documentation/manuals/gnupg/Unattended-GPG-key-generation.html)

```bash
cat >foo <<EOF
%echo Generating a basic OpenPGP key

Key-Type: RSA

Key-Length: 4096
Name-Real: mount-manager

# Do a commit here, so that we can later print "done" :-)
%commit
%echo done
EOF
gpg --batch --generate-key foo
```


### Generate a `gpg` key in interactive mode

If option `--full-generate-key` is not supported try `--full-gen-key` or for very old version `--gen-key`.

```bash
gpg --full-generate-key
```

For mount-manager the required key is for a private and local usage only.

> Please select what kind of key you want:

Select `1`: "(1) RSA and RSA (default)"

> What key size do you want? (2048)

Response `4096`

> Please specify how long the key should be valid.

Select relevant value for you. `key does not expire` is not recommended.

> Real name:

Reply `mount-manager`

Leave other entries empty.

It's probably a *bad idea* to to not set a pass phrase but any way if you
take this option has a first approach you can modify your key and set a
pass phrase at any time using option: `--edit-key`.


### How-to modify your key pass phrase

To add, change or remove your pass phrase you just need to use

```bash
gpg --edit-key mount-manager passwd
```


How-to encrypt your passwords
-----------------------------

To encrypt your password using you gpg key, just run

```bash
mount-manager-encrypt
```

This will generate a line like this:

```json
      "gpg":"-----BEGIN PGP MESSAGE-----^Version: GnuPG v1^^hQIMAznmT ... =uQ1K^-----END PGP MESSAGE-----^",

```

Then just copy this line in the right section of configuration using:

```bash
mount-manager-config
```


Others useful `gpg` commands
-----------------------------

To list the keys in your public key ring:

```bash
gpg --list-keys
```

To list the keys in your secret key ring:

```bash
gpg --list-secret-keys
```

Removes the public key from your public key ring.

```bash
gpg --delete-key "User Name or UID"
```

Delete a private key (a key on your private key ring):

```bash
gpg --delete-secret-key "User Name or UID"
```


References
----------

* <https://help.github.com/articles/generating-a-new-gpg-key/>
* <http://irtfweb.ifa.hawaii.edu/~lockhart/gpg/>

