﻿mount-manager
=============

Installation
------------

Pour commencer, vous devez obtenir le projet. Pour cela, vous pouvez télécharger [la dernière version publiée](https://gitea.cclaude.rocks/tools/mount-manager/releases) (récupérer le fichier **tar.gz**) ou utiliser git:

```bash
git clone --recurse-submodules https://git.cclaude.rocks/tools/mount-manager.git
```

```bash
sudo bash ./mount-manager-install.sh
```

Pour supprimer l’application, utilisez :

```bash
sudo bash /usr/local/share/mount-manager/mount-manager-remove.sh
```

### Options d'installation

Vous pouvez utiliser la syntaxe suivante pour obtenir l’aide.

```bash
bash ./bash_validator-install.sh --help
```

| Option                               | Description |
|--------------------------------------|-------------|
| -h, --help                           | Affiche l’aide et quitte. |
| -S, --skip-update                    | Ignore l’étape de mise à jour du gestionnaire de paquet. |
| -P PMEXE, --package-manager PMEXE    | Force le nom d’exécutable du gestionnaire de paquet |


Les dépendances
---------------

Certaines dépendances sont installées par `mount-manager-install.sh` :

* `jq` est nécessaire pour l’analyse de la configuration au format JSON
* `gpg` si vous utiliser l’encodage des mots de passe (optionnel mais recommandé)
* `open-iscsi` pour le support ISCSI (optionnel)
* `nfs-common` pour le support NFS (optionnel)

En cas de besoin, vous pouvez installer manuellement les paquets suivants :

```bash
sudo apt update && sudo apt upgrade
sudo apt install -y jq
# Optionel: support de l'encodage des mots de passe
sudo apt install -y gpgv2
# Optionel: support du protocole iscsi
sudo apt install -y open-iscsi
# Optional: support du protocole NFS (alpha version)
sudo apt-get install nfs-common
```


Configuration
-------------

Le fichier de configuration se trouve par défaut `~/.config/cclaude.rocks/mount-manager.json`
Pour créer ce fichier utilisez le script `mount-manager-config`.

Notes :

* **Le langage JSON ne supporte la notion de commentaire**. Les commentaires des exemples (commençant par #) ne doivent pas être copié dans le fichier de configuration.
* Le fichier de configuration est au format JSON. Pour respecter la syntaxe JSON le fichier doit, en particulier, commencer par { et finir par }.

### Description du fichier de configuration au format JSON

```json
{
  "options": {
    "@comment": "Ces valeurs sont optionnelles",
    "CHECKLIST_HEIGHT": 20,
    "CHECKLIST_WIDTH": 80,,
    "CHECKLIST_LISTHEIGHT": 10,
  }
  "mount":{
    "diskimage": [              # Optional bloc for image disk (*.dd, *.img, *.iso)
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "samba": [                  # Optional bloc for Samba shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "nfs": [                    # Optional bloc for NFS shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "sshfs": [                  # Optional bloc for sshfs shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "iscsi": [                  # Optional bloc for NFS shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ]
  }
}
```

Exemple de fichier de configuration vide mais valide avec lequel vous pouvez commencer :


```json
{
  "mount": {
    "samba": [],
    "nfs": [],
    "diskimage": [],
    "iscsi": []
  }
}
```

### Samba configuration

Vous pouvez configurer plusieurs serveurs, la clé `samba` attend comme valeur un tableau d’`entrée samba`.

```json
{
  "mount": {
    "samba": [
      {
         ... entrée samba ..
      },
      {
         ... entrée samba ..
      }
    ]
  }
}
```

Une `entrée Samba` est défini à l’aide des clés/valeurs suivantes :

| Clé         | Type   | Description |
|-------------|--------|-------------|
| `id`        | String | Doit être unique (sans espace). |
| `ip`        | String | IP (ou nom DNS) du serveur. |
| `username`  | String | Nom d’utilisateur pour la connexion SAMBA. |
| `password`  | String | mot de passe en clair pour la connexion SAMBA – *laisser vide pour utiliser un mot de passe chiffré*. |
| `gpg`       | Arrays of strings | (optionnel) Permet de définir le mot de passe chiffré. |
| `options`   | String | (optionnel) Options spécifiques pour `mount cifs`. |
| `shares`    | Arrays of strings | Chaine chaîne doit être le nom d'un partage existant. |
| `mount-all` | Boolean | (optionnel) Si la valeur est `true`, les partages seront montés lorsque l’option `--mount-all` or `-M` sera utilisée. |

Utilisation d’un mot de passe chiffré (plus de détail dans <ENCRYPTION-FR.md>) :

```json
    {
      "id": "NAS1",
      "ip": "192.168.10.1",
      "username": "a_user_name",
      "password": "",
      "gpg": [
        "-----BEGIN PGP MESSAGE-----",
        "Version: GnuPG v1",
        "",
        "yWWTp9XiMSlnP/KSF9rhlbWY0dkMajfTTEfGw8qNj/YcqZWoG2fLjPTjV26WWgoe",
        "EYX8Y22WyUL+OKgXGyvasYQVGZB0/ufcL3Ij2nfO4ZP+C9BtwlOhk0ZhiAyNJeLG",
        ...
        "asYQVGZB0/ufcP+C9BtwlOhk0ZhiALEYX8Y22W3Ij2yUL+OKgXnfO4ZyNJeLGGyv",
        "wlOhk0ZhBt3Ij2nfO4ZyiALEYX8Y22WasYQVGZB0/ufcP+C9NJeLGyUL+OKgXGyv",
        "-----END PGP MESSAGE-----"
        ],
      "mount-all": true,
      "shares": [
        "share1",
        "share2",
        "share3"
      ]
    }
```

Utilisation d’un mot de passe en clair :

```json
    {
      "id": "NAS2",
      "ip": "192.168.10.2",
      "username": "a_other_user",
      "password": "PlainTextPassword",
      "gpg": null,
      "shares": [
        "share1",
        "share2"
      ]
    }
  ],
```


### NFS configuration

Not supported yet.


### **sshfs** configuration

Vous pouvez configurer plusieurs serveurs, la clé `sshfs` attend comme valeur un tableau d’`entrée sshfs`.

```json
{
  "mount": {
    "sshfs": [
      {
         ... sshfs entry ..
      },
      {
         ... sshfs entry ..
      }
    ]
  }
}
```

Une `entrée sshfs` est défini à l’aide des clés/valeurs suivantes :

| Clé           | Type   | Description |
|---------------|--------|-------------|
| `id`          | String | Doit être unique (sans espace) |
| `host`        | String | IP (ou nom DNS) du serveur |
| `username`    | String | Nom d’utilisateur pour la connexion ssh |
| `options`     | String | (optionnel) Options spécifiques pour `mount cifs` |
| `directories` | Arrays of objects | Un tableau d'object sshfs_directory |
| `mount-all`   | Boolean | (optionnel) Si la valeur est `true`, les partages seront montés lorsque l’option `--mount-all` or `-M` sera utilisée. |

Un objet `sshfs_directory` est défini à l’aide des clés/valeurs suivantes :

| Clé         | Type   | Description |
|-------------|--------|-------------|
| `directory` | String | Chemin sur le serveur distant du partage |
| `id-suffix` | String | Suffix ID utilisé pour générer le nom de partage local |


### ISCSI configuration

TODO


### Disk image configuration

TODO


Usage
-----

```
mount-manager --help
mount-manager [--verbose]
mount-manager --iscsi-logout [--verbose]
mount-manager --list [--verbose]
mount-manager --list-id [--verbose]
mount-manager --mount MOUNTDIR [--mount MOUNTDIR]... [--umount MOUNTDIR]... [--verbose]
mount-manager --mount-all [--verbose]
mount-manager --mount-default [--verbose]
mount-manager --umount MOUNTDIR [--umount MOUNTDIR]... [--mount MOUNTDIR]... [--verbose]
mount-manager --umount-all [--verbose]
mount-manager --umount-all-force [--verbose]
mount-manager --umount-default [--verbose]
```

### Actions

```
  -h,           --help
                        Affiche l'aide et quitte.

                --iscsi-logout
                        Logout all iSCSI connections and exit

  -l,           --list  List all defined mount directories.

  -L,           --list-id
                        List all defined id.

  -m MOUNTDIR,  --mount MOUNTDIR
                        Mount MOUNTDIR (if defined in mount-manager configuration)

  -M,           --mount-all
                        Mount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true

  -D,           --mount-default
                        Mount every thing define in configuration file with key:
                            "mount-default": true

  -u MOUNTDIR,  --umount MOUNTDIR
                        Umount MOUNTDIR (if mounted)

  -U,           --umount-all
                        Umount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true

  -F,           --umount-all-force
                        Umount every thing define in configuration file, ignore
                        keys "mount-all" and "mount-default" contents.

  -X,           --umount-default
                        Umount every thing define in configuration file with key:
                            "mount-default": true
```

### Options

```
  -v, --verbose         Affiche des informations permettant d'effectuer un diagnostique.
```

### Samples

* Affiche le menu permettant de sélectionner les partages ou les disques à monter ou à démonter

```bash
mount-manager
```

* Monte tous les partages ou les disques définis dans le fichier de configuration avec la clé

```json
"mount-all": true
```

ou

```json
"mount-default": true
```

```bash
mount-manager --mount-all
```

* Démonte tous les partages ou les disques définis dans le fichier de configuration avec la clé

```json
"mount-all": true
```

ou

```json
"mount-default": true
```

```bash
mount-manager --umount-all
```

* Démonte tous les partages ou les disques définis dans le fichier de configuration

```bash
mount-manager --umount-all-force
```


Debug
-----

```bash
tail -f /var/log/kern.log
```

```bash
dmesg -Tw
```


Références
----------

À propos du protocole Samba :

* https://wiki.samba.org/index.php/Mounting_samba_shares_from_a_unix_client

À propos du protocole iSCSI :

* https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/6/html/Storage_Administration_Guide/ch-iscsi.html
* https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/5/html/Online_Storage_Reconfiguration_Guide/iscsilogin.html
* https://linux.die.net/man/8/iscsiadm
* https://wiki.debian.org/SAN/iSCSI/open-iscsi

À propos du protocole NFS:

* https://linux.developpez.com/formation_debian/nfs.html


Références pour le développement
--------------------------------

À propos de Whiptail :

* https://en.wikibooks.org/wiki/Bash_Shell_Scripting/Whiptail
