mount-manager
=============

Installation
------------

To get started, you need to get the project. For this you can download the [latest released version](https://gitea.cclaude.rocks/tools/mount-manager/releases) (get the **tar.gz** file) or use git:

```bash
git clone --recurse-submodules https://git.cclaude.rocks/tools/mount-manager.git
```

```bash
sudo bash ./mount-manager-install.sh
```

To remove the applications, use:

```bash
sudo bash /usr/local/share/mount-manager/mount-manager-remove.sh
```

### Installation options

You can use following command to get help:

```bash
bash ./bash_validator-install.sh --help
```

| Option                               | Description |
|--------------------------------------|-------------|
| -h, --help                           | Display help message and exit. |
| -S, --skip-update                    | Skip package manager update |
| -P PMEXE, --package-manager PMEXE    | Force package manager runtime |


Dependencies
------------

Some dependencies are installed by `mount-manager-install.sh` :

* `jq` is required for json analysis
* `gpg` for password encryption support (optional but recommended)
* `open-iscsi` for iscsi support (optional)
* `nfs-common` for NFS support (optional)

If needed, you can manually install these packages using:

```bash
sudo apt update && sudo apt upgrade
sudo apt install -y jq
# Optional: password encryption support
sudo apt install -y gpgv2
# Optional: iscsi support
sudo apt install -y open-iscsi
# Optional: NFS support (alpha version)
sudo apt-get install nfs-common
```


Configuration
-------------

Configuration file is stored in `~/.config/cclaude.rocks/mount-manager.json`
you can use `mount-manager-config` to edit these file.

Notes:

* **JSON language does not support comments** - So comment should not be use in real file
* A JSON file, it must start by { and finish by }

### Description of JSON configuration file

```json
{
  "options": {
    "comment": "Theses values are optionals",
    "CHECKLIST_HEIGHT": 20,
    "CHECKLIST_WIDTH": 80,,
    "CHECKLIST_LISTHEIGHT": 10,
  }
  "mount":{
    "diskimage": [              # Optional bloc for image disk (*.dd, *.img, *.iso)
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "samba": [                  # Optional bloc for Samba shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "nfs": [                    # Optional bloc for NFS shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "sshfs": [                  # Optional bloc for sshfs shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ],
    "iscsi": [                  # Optional bloc for NFS shares configuration
      { ... First entry ... },  # Optional entry
      { ... Last entry ... }    # Optional entry
    ]
  }
}
```

A valid empty configuration file where you can start :


```json
{
  "mount": {
    "samba": [],
    "nfs": [],
    "diskimage": [],
    "iscsi": []
  }
}
```

### Samba configuration

`samba` attribute expect to have an array on samba entries.

```json
{
  "mount": {
    "samba": [
      {
         ... samba entry ..
      },
      {
         ... samba entry ..
      }
    ]
  }
}
```

A Samba entry is define by

| key        | type   | description |
|------------|--------|-------------|
| `id`       | String | Must be unique (no space) |
| `ip`       | String | IP (or DNS name) of the server |
| `username` | String | User name for SAMBA connection
| `password` | String | plain text password for SAMBA connection - *leave empty if you plan to user an encrypted password*
| `gpg`      | Arrays of strings | (optional if you use a plain text password)
| `options`` | String | where you can customize mount cifs options (optional)
| `shares`   | Arrays of strings | Each string must be a existing share name |
| `mount-all` | Boolean | An optional boolean, if set to `true`, share with be mounted when using `--mount-all` or `-M` option |

Using a encrypted password (more details in <ENCRYPTION.md>)

```json
    {
      "id": "NAS1",
      "ip": "192.168.10.1",
      "username": "a_user_name",
      "password": "",
      "gpg": [
        "-----BEGIN PGP MESSAGE-----",
        "Version: GnuPG v1",
        "",
        "yWWTp9XiMSlnP/KSF9rhlbWY0dkMajfTTEfGw8qNj/YcqZWoG2fLjPTjV26WWgoe",
        "EYX8Y22WyUL+OKgXGyvasYQVGZB0/ufcL3Ij2nfO4ZP+C9BtwlOhk0ZhiAyNJeLG",
        ...
        "asYQVGZB0/ufcP+C9BtwlOhk0ZhiALEYX8Y22W3Ij2yUL+OKgXnfO4ZyNJeLGGyv",
        "wlOhk0ZhBt3Ij2nfO4ZyiALEYX8Y22WasYQVGZB0/ufcP+C9NJeLGyUL+OKgXGyv",
        "-----END PGP MESSAGE-----"
        ],
      "mount-all": true,
      "shares": [
        "share1",
        "share2",
        "share3"
      ]
    }
```

Using a plain text password

```json
    {
      "id": "NAS2",
      "ip": "192.168.10.2",
      "username": "a_other_user",
      "password": "PlainTextPassword",
      "gpg": null,
      "shares": [
        "share1",
        "share2"
      ]
    }
  ],
```


### NFS configuration

Not supported yet.


### **sshfs** configuration

Vous pouvez configurer plusieurs serveurs, la clé `sshfs` attend comme valeur un tableau d’`entrée sshfs`.

```json
{
  "mount": {
    "sshfs": [
      {
         ... sshfs entry ..
      },
      {
         ... sshfs entry ..
      }
    ]
  }
}
```

Une `entrée sshfs` est défini à l’aide des clés/valeurs suivantes :

| Clé           | Type   | Description |
|---------------|--------|-------------|
| `id`          | String | Must be unis (no space) |
| `host`        | String | Server IP (or name) |
| `username`    | String | ssh user name |
| `options`     | String | where you can customize mount-sshfs options (optional) |
| `directories` | Arrays of objects | Un tableau d'object sshfs_directory |
| `mount-all`   | Boolean | An optional boolean, if set to `true`, share with be mounted when using `--mount-all` or `-M` option |

Un objet `sshfs_directory` est défini à l’aide des clés/valeurs suivantes :

| Clé         | Type   | Description |
|-------------|--------|-------------|
| `directory` | String | Remote path |
| `id-suffix` | String | ID suffix user to generat local share name |


### ISCSI configuration

TODO


### Disk image configuration

TODO


Usage
-----

```
mount-manager --help
mount-manager [--verbose]
mount-manager --iscsi-logout [--verbose]
mount-manager --list [--verbose]
mount-manager --list-id [--verbose]
mount-manager --mount MOUNTDIR [--mount MOUNTDIR]... [--umount MOUNTDIR]... [--verbose]
mount-manager --mount-all [--verbose]
mount-manager --mount-default [--verbose]
mount-manager --umount MOUNTDIR [--umount MOUNTDIR]... [--mount MOUNTDIR]... [--verbose]
mount-manager --umount-all [--verbose]
mount-manager --umount-all-force [--verbose]
mount-manager --umount-default [--verbose]
```

### Actions

```
  -h,           --help
                        Display help and exit.

                --iscsi-logout
                        Logout all iSCSI connections and exit

  -l,           --list  List all defined mount directories.

  -L,           --list-id
                        List all defined id.

  -m MOUNTDIR,  --mount MOUNTDIR
                        Mount MOUNTDIR (if defined in mount-manager configuration)

  -M,           --mount-all
                        Mount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true

  -D,           --mount-default
                        Mount every thing define in configuration file with key:
                            "mount-default": true

  -u MOUNTDIR,  --umount MOUNTDIR
                        Umount MOUNTDIR (if mounted)

  -U,           --umount-all
                        Umount every thing define in configuration file with key:
                            "mount-all": true
                        or
                            "mount-default": true

  -F,           --umount-all-force
                        Umount every thing define in configuration file, ignore
                        keys "mount-all" and "mount-default" contents.

  -X,           --umount-default
                        Umount every thing define in configuration file with key:
                            "mount-default": true
```

### Options

```
  -v, --verbose         Show more information (debugging)
```

### Samples

* Display menu to select shares or disjs to mount/umount

```bash
mount-manager
```

* Mount all define shares or disks with key:

```json
"mount-all": true
```

or

```json
"mount-default": true
```

```bash
mount-manager --mount-all
```

* Unmount all define shares or disks with key:

```json
"mount-all": true
```

or

```json
"mount-default": true
```

```bash
mount-manager --umount-all
```

* Unmount all define shares or disks defined in configuration file

```bash
mount-manager --umount-all-force
```


Debug
-----

```bash
tail -f /var/log/kern.log
```

```bash
dmesg -Tw
```


References
----------

About Samba:

* https://wiki.samba.org/index.php/Mounting_samba_shares_from_a_unix_client

About iSCSI:

* https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/6/html/Storage_Administration_Guide/ch-iscsi.html
* https://access.redhat.com/documentation/en-US/Red_Hat_Enterprise_Linux/5/html/Online_Storage_Reconfiguration_Guide/iscsilogin.html
* https://linux.die.net/man/8/iscsiadm
* https://wiki.debian.org/SAN/iSCSI/open-iscsi

About NFS

* https://linux.developpez.com/formation_debian/nfs.html


Developper references
---------------------

About Whiptail:

* https://en.wikibooks.org/wiki/Bash_Shell_Scripting/Whiptail
