#!/bin/bash
#
# mount-manager:bin/api/crypt-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi

declare -r __CRYPT_KEY_ID__='mount-manager'

# Internal
function encrypt_gpg {
  local -r password="$1"

  echo -n "${password}" | gpg --armor --encrypt --batch --recipient "${__CRYPT_KEY_ID__}"
  local -i gpg_rc=$?

  if [ ${gpg_rc} -ne 0 ]; then
    echofatal "pgp error: ${gpg_rc}"
    return ${gpg_rc}
  fi
}

#
# API/DEPRECATED: encrypt <password_to_encrypt>
#
function DEPRECATED_encrypt { # DEPRECATED 2021-10-19
  local -r password_to_encrypt="$1"

  encrypt_gpg "${password_to_encrypt}" | while IFS='' read -r line || [[ -n "${line}" ]]; do
    echo -n "${line}^"
  done
  echo
}

#
# API: encrypt_to_json <password_to_encrypt>
#
function encrypt_to_json {
  local -r password_to_encrypt="$1"

  echo -e "$( encrypt_gpg "${password_to_encrypt}" )" | jq  --raw-input '.' | jq --slurp -c '.'
  echo
}

# Internal
function decrypt_gpg {
  local -r encrypt_gpg_value="$1"

  echo -n "${encrypt_gpg_value}" | gpg --decrypt --batch 2>/dev/null
  local -i gpg_rc=$?

  if [ ${gpg_rc} -ne 0 ]; then
    echofatal "pgp error: ${gpg_rc}"
    return ${gpg_rc}
  fi
}

#
# API/DEPRECATED: decrypt <encrypted_string> # DEPRECATED 2021-10-19
#
function DEPRECATED_decrypt { # But still used...
  local -r encrypted="$1"

  local encrypt_gpg_value=
  encrypt_gpg_value="$( echo "${encrypted}" | cut -d'^' --output-delimiter='
' -f1- )"

  decrypt_gpg "${encrypt_gpg_value}" || return $?
}

#
# API: decrypt_from_json <encrypted_string>
#
function decrypt_from_json {
  local -r encrypted="$1"

  if [ "${encrypted:0:1}" = '[' ] ; then
    # Look like JSON array, so try using jq
    local encrypt_gpg_value=
    encrypt_gpg_value="$( jq -r '.[]' <<<"${encrypted}" )" || return $?
    decrypt_gpg "${encrypt_gpg_value}" || return $?
  else
    # Try previous version
    DEPRECATED_decrypt "${encrypted}" || return $?
  fi
}
