#!/bin/bash
#
# mount-manager:bin/api/diskimage-api.sh
#
if [ "$( type -t echofatal )" != 'function' ] ; then
  echo 'https://git.cclaude.rocks/bash-apis/bash-common-api is required: Function echofatal not found.' >&2
  exit 1
fi

# Internal
function __build_checkboxes_for_diskimage_for_id__ {
  local -r diskimage_id="$1"

  local mountdir=
  mountdir="$( get_diskimage_mountdir "${diskimage_id}" )" || return $?

  local disk_image_file=
  disk_image_file="$( get_diskimage_imagefile "${diskimage_id}" )" || return $?

  update_checkboxes "${mountdir}" "${disk_image_file}" || return $?
}

##
## API: build_checkboxes_for_diskimage
##
function build_checkboxes_for_diskimage {
  echoinfo "${FUNCNAME[0]}()"

  local selected_ids=
  selected_ids="$( get_all_diskimage_ids )" || return $?
  local -r selected_ids

  local selected_id=
  while read -r selected_id ; do
    if [ -n "${selected_id}" ] ; then
      __build_checkboxes_for_diskimage_for_id__ "${selected_id}" || return $?
    fi
  done <<<"${selected_ids}"
}

##
## API: mount_all_diskimage
##
function mount_all_diskimage {
  local id=
  local image=
  local mount_name=
  local mountdir=

  for mountdir in $( get_all_diskimage_mountdir ); do

    mount_name="$( echo "${mountdir}" | rev | cut -d'/' -f1 |rev )"
    id="$( get_diskimage_id_from_mountname "${mount_name}" )"

    if [ -z "${id}" ]; then
      echofatal "Error cannot find disk_image for '${mountdir}'"
    else
      image="$( get_diskimage_imagefile "${id}" )" || return $?
      mount_diskimage "${image}" "${mountdir}" || return $?
    fi

  done
}

##
## API: mount_diskimage_by_mountdir MOUNTDIR
##
function mount_diskimage_by_mountdir {
  local -r mountdir="$1"

  local diskimage_file=
  local diskimage_mountdir=

  for id in $( get_all_diskimage_ids ); do
    if [ -n "${id}" ] ; then
      diskimage_mountdir="$( get_diskimage_mountdir "${id}" )" || return $?

      if [ "${mountdir}" = "${diskimage_mountdir}" ]; then
        diskimage_file="$( get_diskimage_imagefile "${id}" )" || return $?
        break
      fi
    fi
  done

  if [ -z "${diskimage_file}" ]; then
    echofatal "No disk image for '${mountdir}'"
    return 100
  fi

  if [ ! -f "${diskimage_file}" ]; then
    echofatal "Cannot find or access to '${diskimage_file}' for '${mountdir}'"
  else
    mount_diskimage "${diskimage_file}" "${mountdir}" || return $?
  fi
}

##
## mount *.img, *.dd or *.iso files
##
function mount_diskimage {
  local -r disk_image_file="$1"
  local -r mountdir="$2"

  if [ ! -f "${disk_image_file}" ]; then
    echofatal "Cannot find image : '${disk_image_file}'"
    return 100
  fi

  local mounted_status=
  mounted_status="$( get_mounted_status "${mountdir}" )" || return $?
  if [ "${mounted_status}" = 'on' ] ; then
    echowarn "Path '${mountdir}' already mounted."
    return 0
  fi

  create_mountdir_if_needed "${mountdir}" || return $?

  (
    [ "${VERBOSE}" = "true" ] && set -x
    sudo mount -v -o loop,ro "${disk_image_file}" "${mountdir}"
  ) || return $?
}

##
## API: umount_all_diskimage
##
function umount_all_diskimage {
  local mountdir=

  local selected_mountdirs=
  selected_mountdirs="$( get_all_diskimage_mountdir )" || return $?
  local -r selected_mountdirs

  local mountdir=
  while read -r mountdir ; do
    if [ -n "${mountdir}" ] ; then
      umount_by_mountdir "${mountdir}" || true # Ignore errors on umount
    fi
  done <<<"${selected_mountdirs}"
}
