#!/bin/bash
#
# mount-manager:bin/api/nfs-api.sh (ALPHA-VERSION)
#

# Internal
function __build_nfs_sharepath__ {
  local -r server_ip="$1"
  local -r server_share="$2"

  echo "${server_ip}:${server_share}"
}

# Internal
function __mount_nfs__ {
  local -r username="$1"
  local -r password="$2"
  local -r server_ip="$3"
  local -r server_share="$4"
  local -r server_prefix="$5"

  local mountdir=
  mountdir="$( get_nfs_mountdir "${server_prefix}" "${server_share}" )" || return $?

  local mounted_status=
  mounted_status="$( get_mounted_status "${mountdir}" )" || return $?
  if [ "${mounted_status}" = 'on' ] ; then
    echowarn "Path '${mountdir}' already mounted."
    return 0
  fi

  local -r share="${server_ip}:/${server_share}"

  create_mountdir_if_needed "${mountdir}" || return $?

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    sudo mount -vvv -t nfs4 -o "user=${username},password=${password}" "${share}" "${mountdir}"
  ) || return $?
}

##
## API: build_checkboxes_for_nfs
##
function build_checkboxes_for_nfs {
  echoinfo "${FUNCNAME[0]}()"

  local id=
  local ip=
  local mountdir=
  local share_name=
  local share_path=

  for id in $( get_all_nfs_ids ); do
    ip="$( config_json_get_type_value 'nfs' "${id}" 'ip' )" || return $?

    while read -r share_name; do

      mountdir="$( get_nfs_mountdir "${id}" "${share_name}" )" || return $?
      share_path="$( __build_nfs_sharepath__ "${ip}" "${share_name}" )" || return $?

      update_checkboxes "${mountdir}" "${share_path}" || return $?
    done < <( get_nfs_shares "${id}" )
  done
}

##
## API: get_all_nfs_ids
##
function get_all_nfs_ids {
  config_json_get_type_ids 'nfs' || return $?
}

##
## API: get_all_nfs_mountdir
##
function get_all_nfs_mountdir {
  local nfs_id=
  local share=

  for nfs_id in $( get_all_nfs_ids ); do
    while read -r share; do
      # print mount_dir
      get_nfs_mountdir "${nfs_id}" "${share}" || return $?
    done < <( get_nfs_shares "${nfs_id}" )
  done
}

##
## API: get_nfs_mountdir NFS_ID SERVER_SHARE
##
function get_nfs_mountdir {
  local -r nfs_id="$1"
  local -r server_share="$2"

  get_mountdir "${nfs_id}/${server_share}" || return $?
}

##
## API: get_nfs_shares ID
##
function get_nfs_shares {
  local -r id="$1"

  config_json_get_type_value 'nfs' "${id}" 'shares[]' || return $?
}

##
## API: mount_all_nfs
##
function mount_all_nfs {
  local selected_ids=
  selected_ids="$( get_all_nfs_ids )" || return $?
  local -r selected_ids

  local selected_id=
  while read -r selected_id ; do
    if [ -n "${selected_id}" ] ; then
      mount_nfs_by_id "${selected_id}" || return $?
    fi
  done <<<"${selected_ids}"
}

##
## API: mount_nfs_by_id ID
##
function mount_nfs_by_id {
  local -r nfs_id="$1"

  local ip=
  ip="$( config_json_get_type_value 'nfs' "${nfs_id}" 'ip' )" || return $?

  local password=
  password="$( config_json_get_password_by_type_id 'nfs' "${nfs_id}" 'password' )" || return $?

  local shares=
  shares="$( get_nfs_shares "${nfs_id}" )" || return $?

  local username=
  username="$( config_json_get_type_value 'nfs' "${nfs_id}" 'username' )" || return $?

  local share=
  for share in ${shares}; do
    __mount_nfs__ "${username}" "${password}" "${ip}" "${share}" "${nfs_id}" || return $?
  done
}

##
## API: umount_all_nfs
##
function umount_all_nfs {
  local mountdirs=
  mountdirs="$( get_all_nfs_mountdir )" || return $?
  local -r mountdirs

  local mountdir=
  while read -r mountdir ; do
    if [ -n "${mountdir}" ] ; then
      umount_by_mountdir "${mountdir}" || true # Ignore errors on umount
    fi
  done <<<"${mountdirs}"
}

