#!/bin/bash
#
# mount-manager:bin/api/samba-config-json-api.sh
#

# Internal
#
# __samba_flattern_json__ $( jq -c '.mount.samba // [] | .[]' CONFIG_FILE )
#
function __samba_flattern_json__ {
  local -r samba_json_source="$1"

  local samba_flattern_result=
  samba_flattern_result="$(
    local samba_json_entry=
    while read -r samba_json_entry ; do
      if [ -n "${samba_json_entry}" ] ; then
        __samba_flattern_json_entry__ "${samba_json_entry}" || return $?
      fi
    done <<<"${samba_json_source}"
  )" || return $?

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c --slurp --sort-keys '.' <<<"${samba_flattern_result}"
  ) || return $?
}

# Internal
function __samba_flattern_json_entry__ {
  local -r samba_json_entry="$1"

  local shares=
  shares="$( jq -c -r '.shares // [] | .[]' <<<"${samba_json_entry}" )" || return $?
  local -r shares

  local id_source=
  id_source="$( jq -r '.id // empty' <<<"${samba_json_entry}" )" || return $?
  local -r id_source

  local options=
  options="$( jq -r '.options // empty' <<<"${samba_json_entry}" )" || return $?
  local -r options

  local gpg=
  gpg="$( jq -r '.gpg // empty' <<<"${samba_json_entry}" )" || return $?
  local -r gpg

  if [ "${shares:0:1}" = '{' ] ; then
    # Is an object
    local share_object=
    local share=
    local id_suffix=
    while read -r share_object; do
      if [ -n "${share_object}" ] ; then
        share="$( jq -r '.["share-name"] // empty' <<<"${share_object}" )" || return $?
        id_suffix="$( jq -r '.["id-suffix"] // empty' <<<"${share_object}" )" || return $?

        __samba_flattern_rewrite_json_entry__ \
          "${samba_json_entry}" \
          "${id_source}" \
          "${options}" \
          "${gpg}" \
          "${share}" \
          "${id_suffix}" || return $?
      fi
    done <<<"${shares}"
  else
    # Is a string
    local share=
    while read -r share; do
      if [ -n "${share}" ] ; then
        __samba_flattern_rewrite_json_entry__ \
          "${samba_json_entry}" \
          "${id_source}" \
          "${options}" \
          "${gpg}" \
          "${share}" \
          '' || return $?
      fi
    done <<<"${shares}"
  fi
}

# Internal
function __samba_flattern_rewrite_json_entry__ {
  local -r samba_json_entry="$1"
  local -r id_source="$2"
  local -r options="$3"
  local -r gpg="$4"
  local -r share="$5"
  local -r id_suffix="$6"

  local full_id=
  if [ -n "${id_suffix}" ] ; then
    full_id="${id_source}.${share}"
  else
    full_id="${id_source}/${share}"
  fi

  local mountpoint=
  mountpoint="$( get_mountdir "${full_id}" )" || return $?
  local -r mountpoint

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c \
      --arg 'fullid' "${full_id}" \
      --arg 'gpg' "${gpg}" \
      --arg 'mountpoint' "${mountpoint}" \
      --arg 'options' "${options}" \
      --arg 'share' "${share}" \
      --arg 'sourceid' "${id_source}" \
      --arg 'sourceidsuffix' "${id_suffix}" \
      'del(.shares) + {
  "full-id": $fullid,
  "gpg": $gpg,
  "id": $fullid,
  "mount-point": $mountpoint,
  "options": $options,
  "source-id": $sourceid,
  "source-id-suffix": $sourceidsuffix,
  "source-share": $share
}' <<<"${samba_json_entry}"
  ) || return $?
}

##
## API: get_all_samba_ids
##
function get_all_samba_ids {
  config_json_get_type_ids 'samba-flattern' || return $?
}

##
## API: get_all_samba_mountdir
##
function get_all_samba_mountdir {
  config_json_get_object -r '.mount | .["samba-flattern"] // [] | .[] | .["mount-point"] // empty' || return $?
}

##
## API: get_samba_mountdir SAMBA_FULLID
##
function get_samba_mountdir {
  local -r samba_fullid="$1"

  config_json_get_type_value 'samba-flattern' "${samba_fullid}" 'mount-point' || return $?
}

##
## API: get_samba_optional_options SAMBA_ID
##
function get_samba_optional_options {
  local -r samba_id="$1"

  config_json_get_type_value 'samba-flattern' "${samba_id}" 'options' || return $?
}

##
## samba_flattern_config <CONFIG_FILE
##
function samba_flattern_config {
  local config_source_json=
  config_source_json="$( jq -c '.' )" || return $?
  local -r config_source_json

  local samba_source_json=
  samba_source_json="$( jq -c '.mount.samba // [] | .[]' <<<"${config_source_json}" )" || return $?
  local -r samba_source_json

  local samba_result_json=
  samba_result_json="$( __samba_flattern_json__ "${samba_source_json}" )" || return $?
  local -r samba_result_json

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c \
      --argjson 'newsamba' "${samba_result_json}" \
      'del( .mount.samba ) | .mount."samba-flattern" |= $newsamba' <<<"${config_source_json}"
  ) || return $?
}

