#!/bin/bash
#
# mount-manager:bin/api/sshfs-config-json-api.sh
#
declare -gr SSHF_ID_SEPARATOR_CHAR='@'
declare -gr SSHF_ID_PATH_SEPARATOR_CHAR='+'

# Internal
#
# __sshfs_flattern_json__ $( jq -c '.mount.sshfs // [] | .[]' CONFIG_FILE )
#
function __sshfs_flattern_json__ {
  local -r sshfs_json_source="$1"

  local sshfs_flattern_result=
  sshfs_flattern_result="$(
    local sshfs_json_entry=
    while read -r sshfs_json_entry ; do
      if [ -n "${sshfs_json_entry}" ] ; then
        __sshfs_flattern_json_entry__ "${sshfs_json_entry}" || return $?
      fi
    done <<<"${sshfs_json_source}"
  )" || return $?

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c --slurp --sort-keys '.' <<<"${sshfs_flattern_result}"
  ) || return $?
}

# Internal
function __sshfs_flattern_json_entry__ {
  local -r sshfs_json_entry="$1"

  local json_directories=
  json_directories="$( jq -c '.directories // [] | .[]' <<<"${sshfs_json_entry}" )" || return $?
  local -r json_directories

  local id_source=
  id_source="$( jq -r '.id // empty' <<<"${sshfs_json_entry}" )" || return $?
  local -r id_source

  local options=
  options="$( jq -r '.options // empty' <<<"${sshfs_json_entry}" )" || return $?
  local -r options

  local json_directory=
  while read -r json_directory; do
    if [ -n "${json_directory}" ] ; then
      __sshfs_flattern_rewrite_json_entry__ \
        "${sshfs_json_entry}" \
        "${id_source}" \
        "${options}" \
        "${json_directory}" || return $?
    fi
  done <<<"${json_directories}"
}

# Internal
function __sshfs_flattern_rewrite_json_entry__ {
  local -r sshfs_json_entry="$1"
  local -r id_source="$2"
  local -r options="$3"
  local -r json_directory="$4"

  local id_suffix=
  id_suffix="$( jq -r '.["id-suffix"] // empty' <<<"${json_directory}" )" || return $?
  local -r id_suffix

  local directory_json=
  directory_json="$( jq '.directory' <<<"${json_directory}" )" || return $?
  local -r directory_json

  if [ "${directory_json}" = 'null' ] ; then
    echofatal "${FUNCNAME[0]}() - missing or null .directory - id_source=${id_source}"
    return 100
  fi
  local directory=
  directory="$( jq -r '.directory // empty' <<<"${json_directory}" )" || return $?
  local -r directory

  local dir_suffix=
  if [ -n "${id_suffix}" ] ; then
    dir_suffix="${id_suffix}"
  else
    local encoded_dir=
    encoded_dir="$( tr '/' "${SSHF_ID_PATH_SEPARATOR_CHAR}" <<<"${directory}" )" || return $?

    local host=
    host="$( jq -r '.host // empty' <<<"${sshfs_json_entry}" )" || return $?

    local username=
    username="$( jq -r '.username // empty' <<<"${sshfs_json_entry}" )" || return $?

    dir_suffix="${host}${SSHF_ID_SEPARATOR_CHAR}${username}${SSHF_ID_SEPARATOR_CHAR}${encoded_dir}"
  fi
  local -r dir_suffix

  local -r full_id="${id_source}${SSHF_ID_SEPARATOR_CHAR}${dir_suffix}"

  local mountpoint=
  mountpoint="$( get_sshfs_mountdir_from_fullid "${full_id}" )" || return $?
  local -r mountpoint

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c \
      --arg 'directory' "${directory}" \
      --arg 'fullid' "${full_id}" \
      --arg 'mountpoint' "${mountpoint}" \
      --arg 'options' "${options}" \
      --arg 'sourceid' "${id_source}" \
      --arg 'sourceidsuffix' "${id_suffix}" \
      'del(.directories) + {
  "full-id": $fullid,
  "id": $fullid,
  "mount-point": $mountpoint,
  "options": $options,
  "source-directory": $directory,
  "source-id": $sourceid,
  "source-id-suffix": $sourceidsuffix
}' <<<"${sshfs_json_entry}"
  ) || return $?
}

##
## API: get_all_sshfs_ids
##
function get_all_sshfs_ids {
  config_json_get_type_ids 'sshfs-flattern' || return $?
}

##
## API: get_all_sshfs_mountdir
##
function get_all_sshfs_mountdir {
  config_json_get_object -r '.mount | .["sshfs-flattern"] // [] | .[] | .["mount-point"] // empty' || return $?
}

##
## API: get_sshfs_mountdir_from_fullid ID
##
function get_sshfs_mountdir_from_fullid {
  local -r sshfs_fullid="$1"

  # print mount_dir
  get_sshfs_mountdir "${sshfs_fullid}" || return $?
}

##
## API: get_sshfs_optional_options ID
##
function get_sshfs_optional_options {
  local -r sshfs_id="$1"

  config_json_get_type_value 'sshfs-flattern' "${sshfs_id}" 'options' || return $?
}

##
## sshfs_flattern_config <CONFIG_FILE
##
function sshfs_flattern_config {
  local config_source_json=
  config_source_json="$( jq -c '.' )" || return $?
  local -r config_source_json

  local sshfs_source_json=
  sshfs_source_json="$( jq -c '.mount.sshfs // [] | .[]' <<<"${config_source_json}" )" || return $?
  local -r sshfs_source_json

  local sshfs_result_json=
  sshfs_result_json="$( __sshfs_flattern_json__ "${sshfs_source_json}" )" || return $?
  local -r sshfs_result_json

  (
    [ "${VERBOSE:-}" = 'true' ] && set -x
    jq -c \
      --argjson 'newsshfs' "${sshfs_result_json}" \
      'del( .mount.sshfs ) | .mount."sshfs-flattern" |= $newsshfs' <<<"${config_source_json}"
  ) || return $?
}
